@@ -192,7 +192,7 @@ static int validate_nla(struct nlattr *nla, int maxtype,
 	unsigned int minlen = 0;
 	int type = nla_type(nla);
 
-	if (type <= 0 || type > maxtype)
+	if (type < 0 || type > maxtype)
 		return 0;
 
 	pt = &policy[type];
@@ -250,10 +250,6 @@ int nla_parse(struct nlattr *tb[], int maxtype, struct nlattr *head, int len,
 	nla_for_each_attr(nla, head, len, rem) {
 		int type = nla_type(nla);
 
-		/* Padding attributes */
-		if (type == 0)
-			continue;
-
 		if (type > maxtype)
 			continue;
 
@@ -652,9 +648,10 @@ int nla_put_u64(struct nl_msg *msg, int attrtype, uint64_t value)
  */
 uint64_t nla_get_u64(struct nlattr *nla)
 {
-	uint64_t tmp;
+	uint64_t tmp = 0;
 
-	nla_memcpy(&tmp, nla, sizeof(tmp));
+	if (nla && nla_len(nla) >= sizeof(tmp))
+		memcpy(&tmp, nla_data(nla), sizeof(tmp));
 
 	return tmp;
 }
